## dotenv-flow-webpack

A secure webpack plugin that supports [dotenv-flow](https://github.com/kerimdzhanov/dotenv-flow) and other environment variables and **only exposes what you choose and use**.

[![npm version](https://badge.fury.io/js/dotenv-flow-webpack.svg)](https://badge.fury.io/js/dotenv-flow-webpack)
[![Build Status](https://travis-ci.org/hk-labs/dotenv-flow-webpack.svg?branch=master)](https://travis-ci.org/hk-labs/dotenv-flow-webpack)
[![dependencies status](https://david-dm.org/hk-labs/dotenv-flow-webpack/status.svg)](https://david-dm.org/hk-labs/dotenv-flow-webpack)


### Installation

Using NPM:

```bash
$ npm install dotenv-flow-webpack --save-dev
```

Using Yarn:

```bash
$ yarn add dotenv-flow-webpack --dev
```


### Description

`dotenv-flow-webpack` wraps `dotenv-flow` as a `Webpack.DefinePlugin`.
Thus, when you set it up to your webpack config it does a text replace in the resulting bundle for any entrance of the `process.env.*`.

Your `.env*` files can include sensitive information.
Because of this, `dotenv-flow-webpack` will only expose environment variables that are **explicitly referenced in your code** to your final bundle.


### Usage

The plugin can be installed with little-to-no configuration needed. Once installed, you can access the variables within your code using `process.env.*` as you would with `dotenv-flow`.

The example bellow shows a standard use-case.

###### Create `.env*` files

```
// .env
APP_ROOT_PATH=/
REST_API=/api/v1

DB_HOST=127.0.0.1
DB_PASS=defaultdbpass

S3_API=mysecretkey
```

```
// .env.local
DB_PASS=mylocaldbpass
```

```
// .env.development
APP_HOST=localhost
APP_PORT=3000

DB_NAME=my_app_development
```

```
// .env.test
APP_HOST=localhost
APP_PORT=3001

DB_NAME=my_app_test
```

```
// .env.production
APP_HOST=www.myapp.com
APP_PORT=

DB_NAME=my_app_production
```

```
// .env.production.local
DB_PASS=myproductiondbpass
```

###### Update your webpack config files

```javascript
// webpack.config.js
const DotenvFlow = require('dotenv-flow-webpack');

module.exports = {
  ...
  plugins: [
    new DotenvFlow()
  ]
  ...
};
```

###### Use in your code

```javascript
// file1.js
console.log('APP_HOST:', process.env.APP_HOST);
console.log('APP_PORT:', process.env.APP_PORT);
console.log('REST_API:', process.env.REST_API);
console.log('DB_HOST:', process.env.DB_HOST);
console.log('DB_NAME:', process.env.DB_NAME);
```

###### Resulting bundle

When building with `NODE_ENV=development`, the resulting `bundle.js` will contain:

```javascript
// bundle.js
console.log('APP_HOST:', 'localhost');
console.log('APP_PORT:', '3000');
console.log('REST_API:', '/api/v1');
console.log('DB_HOST:', '127.0.0.1');
console.log('DB_NAME:', 'my_app_development');
```

And if we build with `NODE_ENV=production`, the resulting `bundle.js` will contain:

```javascript
// bundle.js
console.log('APP_HOST:', 'www.myapp.com');
console.log('APP_PORT:', '');
console.log('REST_API:', '/api/v1');
console.log('DB_HOST:', '127.0.0.1');
console.log('DB_NAME:', 'my_app_production');
```

Note: the values for `DB_PASS` and  `S3_API` are **NOT** present in our bundle, as they were never referenced (as `process.env.[VAR_NAME]`) in the code.


#### How Secure?

By allowing you to define exactly where you are loading environment variables from, and bundling only variables in your project that are explicitly referenced in your code, you can be sure that only what you need is included and you do not accidentally leak anything sensitive.


#### Environment-specific files

Please refer to the dotenv-flow's [NODE_ENV-specific files](https://github.com/kerimdzhanov/dotenv-flow#node_env-specific-env-files) section to learn more about `.env.development`, `.env.test` and `.env.production` environment files.


#### Files under version control

Recommendations on which of `.env*` files to track/ignore are also available [here](https://github.com/kerimdzhanov/dotenv-flow#files-under-version-control).


#### Variables overwriting/priority

As you might have noticed from the example above, variables from multiple env files are merged together and can be overwritten depending on the given environment (`NODE_ENV`).
Refer the dotenv-flow's [variables overwriting/priority](https://github.com/kerimdzhanov/dotenv-flow#variables-overwritingpriority) section to learn more about it.


### Options

Use the following options to configure your instance.

* **environment** (`'development'`) - The dotenv-flow environment (might be any of `'development'`, `'test'`, `'production'`, `'staging'`, etc.).
* **path** (`process.cwd()`) - The path to a directory where `.env*` files are located.
* **encoding** (`'utf8'`) - The encoding of the `.env*` files to parse.
* **systemvars** (`false`) - Set to true if you would rather load all system variables as well (useful for CI purposes).
* **silent** (`false`) - If true, all warnings will be suppressed.

The following example shows how to set any/all arguments.

```javascript
module.exports = {
  ...
  plugins: [
    new DotenvFlow({
      environment: 'production', // build in the production environment
      path: '/path/to/my/project', // load `.env*` files from this directory
      encoding: 'latin1', // set encoding for parsing `.env*` files
      systemvars: true, // load all the predefined 'process.env' variables which will trump anything local per dotenv specs
      silent: true // hide any errors
    })
  ]
  ...
};
```

### LICENSE

MIT
